
#ifndef __CONSOLE_H
#define __CONSOLE_H

namespace con {

class CChannel;
class IConsoleRenderingContext;
class IFilter;


// --------------------------------------------------------------------------

class CConsole {
public:
	static CConsole* const getConsole() {
		if( !mSingleInstance ) mSingleInstance = new CConsole();
		return mSingleInstance;
	}
	static CChannel& getChannel( const std::string& name ) {
		return getConsole()->openChannel( name );
	}

public:
	virtual ~CConsole();

	virtual CChannel& openChannel( const std::string& name );
	virtual void closeChannel( CChannel& channel );

	void setDefaultFilter( IFilter& filter ) { mDefaultFilter = &filter; };
	IFilter* getDefaultFilter() const { return mDefaultFilter; };		

	void setDefaultRenderingContext( IConsoleRenderingContext& ctx );
	IConsoleRenderingContext* getDefaultRenderingContext() const { return mDefaultRenderingCtx; };
	
protected:
	CConsole();
	CConsole( IConsoleRenderingContext& ctx );
	CConsole( IConsoleRenderingContext& ctx, IFilter& filter );

protected:
	typedef std::list<CChannel*>	TChannelList;

private:
	IConsoleRenderingContext*	mDefaultRenderingCtx;
	IFilter*					mDefaultFilter;
	TChannelList				mChannels;

	static CConsole*			mSingleInstance;
};


// --------------------------------------------------------------------------

class CChannel {
public:
	CChannel( CConsole& console, const std::string& name, IConsoleRenderingContext* renderingCtx = NULL, IFilter* filter = NULL );
	virtual ~CChannel();

	virtual void close() { mParentConsole.closeChannel( *this ); }

	virtual void write( const std::string& methodName, const std::string& message );
	virtual void write( const std::string& message );

	void setRenderingContext( IConsoleRenderingContext& ctx ) { mRenderingCtx = &ctx; };
	IConsoleRenderingContext* getRenderingContext() const { return mRenderingCtx; };

	void setFilter( IFilter& filter ) { mFilter = &filter; };
	IFilter* getFilter() const { return mFilter; };

	const std::string& getName() const { return mName; };

	bool isWritingChannel() const { return mWritingChannel; }
	void setWritingChannel( bool wc ) { mWritingChannel = wc; }

	bool isLoggingTime() const { return mLoggingTime; }
	void setLoggingTime( bool wc ) { mLoggingTime = wc; }

	// output operators

	CChannel& endl() { write( buf); buf.erase(); return *this; };
	CChannel& operator <<( CChannel& (__cdecl * _f)(CChannel&) ){  (*_f)(*this); return *this; };
	CChannel& operator <<( const std::string& msg ) { buf += msg; return *this; }
	CChannel& operator <<( const char* msg ) { buf += msg; return *this; }
	CChannel& operator <<( int num ) { char msg[50]; itoa(num, msg, 10); buf += msg; return *this; }
	CChannel& operator <<( char c ) { buf += c; return *this; }
	CChannel& operator <<( DWORD num ) { char msg[50]; sprintf(msg,"%u",num); buf += msg; return *this; }
	CChannel& operator <<( float num ) { char msg[50]; sprintf(msg,"%g",num); buf += msg; return *this; }
	CChannel& operator <<( double num ) { char msg[50]; sprintf(msg,"%g",num); buf += msg; return *this; }
	CChannel& operator <<( D3DXCOLOR const& col ) { char msg[100]; sprintf( msg, "(%01.2f %01.2f %01.2f %01.2f)", col.r, col.g, col.b, col.a ); buf += msg; return *this; }
	CChannel& operator <<( D3DXVECTOR3 const& vec ) { char msg[100]; sprintf( msg, "(%01.2f %01.2f %01.2f)", vec.x, vec.y, vec.z ); buf += msg; return *this; }
	CChannel& operator <<( D3DXVECTOR4 const& vec ) { char msg[100]; sprintf( msg, "(%01.2f %01.2f %01.2f %01.2f)", vec.x, vec.y, vec.z, vec.w ); buf += msg; return *this; }
	CChannel& operator <<( D3DXQUATERNION const& q ) { char msg[100]; sprintf( msg, "(%01.2f %01.2f %01.2f %01.2f)", q.x, q.y, q.z, q.w ); buf += msg; return *this; }
	CChannel& operator <<( D3DXMATRIX const& m ) { char msg[200]; sprintf( msg,
		"(%01.2f %01.2f %01.2f %01.2f | %01.2f %01.2f %01.2f %01.2f | %01.2f %01.2f %01.2f %01.2f | %01.2f %01.2f %01.2f %01.2f)\n",
		m._11, m._12, m._13, m._14,
		m._21, m._22, m._23, m._24,
		m._31, m._32, m._33, m._34,
		m._41, m._42, m._43, m._44 ); buf += msg; return *this; }

protected:
	CConsole&		mParentConsole;
	std::string		buf;

private:
	std::string					mName;
	IConsoleRenderingContext*	mRenderingCtx;
	IFilter*					mFilter;
	bool						mWritingChannel;
	bool						mLoggingTime;
};

inline CChannel& __cdecl flush(CChannel& cc) { return cc.endl(); }
inline CChannel& __cdecl endl (CChannel& cc) { return cc.endl(); }


// --------------------------------------------------------------------------

class IConsoleRenderingContext {
public:
	virtual void write( const std::string& message ) = 0;
};


// --------------------------------------------------------------------------

class IFilter {
public:
	virtual bool apply( const CChannel& channel, const std::string& message ) const = 0;
};


// --------------------------------------------------------------------------

template<class _V, class _SEP = std::string>
class COutputIterator {
public:
	typedef COutputIterator<_V, _SEP>	this_type;

public:
	COutputIterator( CChannel& chn, _SEP const& sep ) : mSeparator(sep), mChannel(chn) {}

	template<typename T>
		COutputIterator const& operator=( T const& p ) { mChannel << p << mSeparator; return *this; }
	this_type& operator*() { return *this; }
	this_type& operator++() { return *this; }
	this_type& operator++( int ) { return *this; }

private:
	CChannel&	mChannel;
	_SEP const&	mSeparator;	
};


} // namespace con

#endif
